﻿#include "game.h"
#include "tiles.h"
#include "walls.h"
#include "doors.h"
#include "player.h"
#include "../common/sources/system/fileCache.h"
#include "../common/sources/system/cgraphics2d.h"
#include <QDebug>
#include <QPainter>
#include "interface.h"
#include "bridge.h"
#include "keyboard.h"

CGame   game;

//-----------------------------------------------------------------------------------------
CGame::CGame()
{
}

//-----------------------------------------------------------------------------------------
void CGame::drawWall(QImage* image, CTile* tile, CVec2 tablePos, EWallSide side, bool flip)
{
    EWallSide playerSide = player.getWallSide(side);

    if (tile->mWalls[playerSide].getType() != eWallNothing)
    {
        const char* fileName = walls.getFilename(tablePos, side, flip);

        if (fileName[0] != 0)
        {
            // dessine le mur
            QImage  wallGfx = fileCache.getImage(fileName);
            CVec2 gfxPos = walls.getGfxPos(tablePos, side, flip);

            graph2D.drawImage(image, gfxPos, wallGfx, 0, flip);

            // dessine les décors
            CWall*  wall = &tile->mWalls[playerSide];

            if (wall->getType() == eWallSimple)
            {
                //------------------------------------------------------------------------------
                // mur simple
                int ornate = wall->getOrnateParam("Decor");
                walls.drawOrnate(image, tablePos, side, ornate);
            }
            else if (wall->getType() == eWallPortrait)
            {
                //------------------------------------------------------------------------------
                // miroir de champion
                walls.drawOrnate(image, tablePos, side, ORNATE_MIRROR);

                if (tablePos == CVec2(2, 3) && side == eWallSideUp)
                {
                    // si le miroir est juste en facede nous, on dessine la tête du champion
                    int champion = wall->getIntParam("Champion");
                    bool isEmpty = wall->getBoolParam("estVide");

                    if (isEmpty == false)
                    {
                        CRect   rect = interface.getChampionPortraitRect(champion);
                        QImage  portraits = fileCache.getImage("gfx/interface/ChampPortraits.png");
                        CVec2   pos(96, 68);
                        graph2D.drawImageAtlas(image, pos, portraits, rect);

                        int lastChampChosed;
                        for (lastChampChosed = 0; lastChampChosed < 4; ++lastChampChosed)
                            if (characters[lastChampChosed].portrait == -1)
                                break;

                        if (lastChampChosed != 4)
                        {
                            CRect   mouseRect(pos, CVec2(pos.x + CHAMP_PORTRAIT_WIDTH - 1, pos.y + CHAMP_PORTRAIT_HEIGHT - 1));
                            mouse.addArea(eMouseAreaG_Champion, mouseRect, eCursor_Hand, (void*)wall);
                        }
                    }
                }
            }
            else if (wall->getType() == eWallDoorSide)
            {
                //------------------------------------------------------------------------------
                // coté d'une porte
                if (tablePos == CVec2(2, 3) && side == eWallSideUp)
                {
                    QImage  doorFrame = fileCache.getImage("gfx/3DView/doors/Frame_Face_2.png");
                    graph2D.drawImage(image, CVec2(96, 33), doorFrame);
                }
            }
        }
    }
}

//-----------------------------------------------------------------------------------------
void CGame::displayMainView(QImage* image)
{
    if (interface.isInInventory() == false)
    {
        CRect   mainRect(CVec2(0, 33), CVec2(MAINVIEW_WIDTH - 1, 33 + MAINVIEW_HEIGHT - 1));
        mouse.addArea(eMouseArea_None, mainRect, eCursor_Hand);

		// dessine le sol et le plafond
        QImage  floorGfx = fileCache.getImage("gfx/3DView/Floor.png");
        QImage  ceilingGfx = fileCache.getImage("gfx/3DView/Ceiling.png");

        bool flip = (player.pos.x + player.pos.y + player.dir) & 1;

        graph2D.drawImage(image, CVec2(0, 33), ceilingGfx, 0, flip);
        graph2D.drawImage(image, CVec2(0, 72), floorGfx, 0, flip);

		// dessine les murs
        CVec2   tablePos;
        CVec2   localPos;
        CVec2   mapPos;

        for (int y = 0; y < WALL_TABLE_HEIGHT; ++y)
            for (int x = 0; x < WALL_TABLE_WIDTH; ++x)
            {
                tablePos = walls.getDrawSequence(CVec2(x, y));
                localPos = walls.getMapPos(tablePos);
                mapPos = player.getPosFromLocal(localPos);
                CTile*  tile = map.getTile(mapPos);

                if (tile != NULL)
                {
                    // dessine les murs
                    drawWall(image, tile, tablePos, eWallSideUp, flip);
                    drawWall(image, tile, tablePos, eWallSideLeft, flip);
                    drawWall(image, tile, tablePos, eWallSideRight, flip);

                    // dessine les décors au sol
                    if (tile->getType() == eTileGround)
                    {
                        //------------------------------------------------------------------------------
                        // case de sol
                        int ornate = tile->getOrnateParam("Decor");
                        tiles.drawFloorOrnate(image, tablePos, ornate);
                    }
                    else if (tile->getType() == eTilePressPlate)
                    {
                        //------------------------------------------------------------------------------
                        // plaque de pression
                        int ornate = tile->getEnumParam("Type");

                        if (ornate != 0)
                            tiles.drawFloorOrnate(image, tablePos, ornate + 6);
                    }

                    // dessine les objets de la ligne arrière

                    // dessine les portes
                    if (tile->getType() == eTileDoor)
                    {
                        doors.draw(image, tile, tablePos);
                    }

                    // dessine les objets de la ligne avant
                    // dessine les ennemis
                }
            }
    }
}

//-----------------------------------------------------------------------------------------
QImage& CGame::mainLoop()
{
    static QImage image(SCREEN_WIDTH,
                 SCREEN_HEIGHT,
                 QImage::Format_RGBA8888);

    mouse.clearAreas();

    // efface l'écran
    image.fill(QColor(0, 0, 0));

    // affichage
    displayMainView(&image);
    interface.display(&image);

    // logique du jeu
    mouse.update();
    SMouseArea* clickedArea = mouse.clickedArea(&image);

    keyboard.update();
    update(clickedArea);
    interface.update(clickedArea);

    return image;
}

//-----------------------------------------------------------------------------------------
void CGame::update(SMouseArea* clickedArea)
{
    doors.update();
    tiles.updatePressPlates();

    if (clickedArea != NULL)
    {
        if (mouse.mButtonPressing == true)
        {
            if (clickedArea->type == eMouseAreaG_Champion)
            {
                lastMirrorClicked = (CWall*)clickedArea->param1;

                int lastChampChosed;
                for (lastChampChosed = 0; lastChampChosed < 4; ++lastChampChosed)
                    if (characters[lastChampChosed].portrait == -1)
                        break;

                characters[lastChampChosed].fromDB(lastMirrorClicked->getIntParam("Champion"));
                interface.enterResurrect(lastChampChosed);
            }
            else if (clickedArea->type == eMouseAreaG_DoorButton)
            {
                CTile*  tile = (CTile*)clickedArea->param1;
                bool isOpened = tile->getBoolParam("estOuverte");
                tile->setBoolParam("estOuverte", !isOpened);
            }
        }
    }
}
